#!/bin/bash
set -x

test_result=
revision=
source_branch=
merge_proposal=
buildjob_url=
ppa_target=
packaging_branch=
TOOLS_PATH='/var/lib/jenkins/ps-qa-tools/scripts/launchpad-trigger'

while getopts "r:v:s:m:b:t:p:a:" opt; do
    case $opt in
    p)
        echo "(p)pa target: $OPTARG" >&2
        ppa_target=$OPTARG
        ;;
    a)
        echo "p(a)ckaging branch: $OPTARG" >&2
        packaging_branch=$OPTARG
        ;;
    r)
        echo "test (r)esult: $OPTARG" >&2
        test_result=$OPTARG
        ;;
    v)
        echo "re(v)ision: $OPTARG" >&2
        revision=$OPTARG
        ;;
    s)
        echo "(s)ource branch: $OPTARG" >&2
        source_branch=$OPTARG
        ;;
    m)
        echo "(m)erge proposal: $OPTARG" >&2
        merge_proposal=$OPTARG
        ;;
    b)
        echo "(b)uild job url: $OPTARG" >&2
        buildjob_url=$OPTARG
        ;;
    t)
        echo "(t)ool-set path: $OPTARG" >&2
        TOOLS_PATH=$OPTARG
        ;;

    \p)
        echo "Invalid option: -$OPTARG" >&2
        exit 1
        ;;
    :)
        echo "Option -$OPTARG requires an argument." >&2
        exit 1
        ;;
    esac
done

#if one of $ppa_target or $packaging_branch is set (the first if)
#then both of them must be set (the second if)
if [ -n "${ppa_target}${packaging_branch}" ]; then
    if [ -z "$ppa_target" ] || [ -z "$packaging_branch" ]; then
        echo "ERROR: you must specify both -p and -a"
        exit 1
    fi
fi

if [ "$test_result" == "FAILED" ]; then
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${buildjob_url}"\
            --landing-failed --revision=${revision} --vote=Disapprove
    exit 1
fi

TMP_FILE=`mktemp`

#the following code should set the following variables
#commit_message=
#bugs_fixed=
#approved_by=
#launchpad_user=
$TOOLS_PATH/getMergeProposalAttributes.py --merge-proposal="${merge_proposal}" \
        --branch=${source_branch} | grep -v '^DEBUG' > "$TMP_FILE"
. "$TMP_FILE"
cat "$TMP_FILE"
rm -f "$TMP_FILE"

if [ -z "$commit_message" ]; then
    echo "Commit message empty."
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${BUILD_URL}"\
            --commit-message-empty --revision="${revision}" --vote=Disapprove
    exit 1
fi

workdir="$WORKSPACE/merging"

result=0

rm -rf "$workdir"
mkdir -p "$workdir"

bzr branch "$target_branch" "$workdir/trunk" || result=1
cd "$workdir/trunk"
bzr merge "$source_branch" || result=1

if [ "$result" == 1 ]; then
    echo "Merging failed"
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${BUILD_URL}" \
	    --merge-failed --revision="${revision}" --vote=Disapprove
    exit 1
fi


#TODO
#bzr commit --author "" --fixes "" --message "<commit_message>. Fixes: <bugs_fixed>. Approved by <approved_by>."
echo "$authors" "$bugs_fixed_ids" | xargs bzr commit --message "$commit_message. Fixes: $bugs_fixed. Approved by $approved_by." || result=1

merge_revision=`bzr revno`

target_branch=`echo "$target_branch" | sed "s,^lp:,bzr+ssh://$launchpad_user@bazaar.launchpad.net/+branch/,"`
bzr push "$target_branch" || result=1

if [ "$result" == 1 ]; then
    echo "Pushing failed"
    $TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}" \
            --merge-proposal="${merge_proposal}" --job-url="${BUILD_URL}"\
            --push-failed --revision="${revision}" --vote=Disapprove
    exit 1
fi

#if we have a ppa, then do a dput
if [ -n "$ppa_target" ]; then

    #first we need to merge the packaging branch and create a source package
    bzr merge "$packaging_branch" || result=1

    # bump the revision to next packaging version and create a source tarball
    export DEBFULLNAME='Martin Mrazik QA'
    export DEBEMAIL=martin.mrazik+qa@canonical.com
    version=`dpkg-parsechangelog | awk '/^Version/ {print $2}' | sed -e "s/\(.*\)-[0-9]ubuntu.*/\1/"`+bzr${merge_revision}
    version=${version}ubuntu0 #+${packaging_rev}
    sourcename=`dpkg-parsechangelog | awk '/^Source/ {print $2}'`
    dch -v $version "Automatic build of revision $merge_revision" #-D ${pocket}
    cd ..
    tar -czf ${sourcename}_${version}.orig.tar.gz trunk
    cd trunk

    debuild -S -d || result=1
    cd ..
    dput "$ppa_target" *_source.changes || result=1
fi

rm -rf "$workdir"

#TODO
$TOOLS_PATH/updateMergeProposal.py --branch="${source_branch}"\
        --merge-proposal="${merge_proposal}" --merged \
        --revision="${revision}" --vote=Approve

$TOOLS_PATH/closeBugs.py  --branch="$source_branch" \
	--merge-proposal="$merge_proposal" 

exit $result
