#!/usr/bin/env python

import getopt, sys
from settings import *
from launchpad import LaunchpadAgent

from bzrlib.plugin import load_plugins
load_plugins()
from bzrlib.branch import Branch


def usage():
    print """usage:
             voteOnMergeProposal.py -wa
            """

def get_authors(mp):
    source_branch = Branch.open('lp:'+mp.source_branch.unique_name)
    target_branch = Branch.open('lp:'+mp.target_branch.unique_name)
    source_branch.lock_read()
    target_branch.lock_read()

    graph = source_branch.repository.get_graph(target_branch.repository)
    unique_ids = graph.find_unique_ancestors(source_branch.last_revision(), [target_branch.last_revision()])

    revs = source_branch.repository.get_revisions(unique_ids)

    author_list = []

    for rev in revs:
        apparent_authors = rev.get_apparent_authors()
        for author in apparent_authors:
                author.replace('\n','')
                if author not in author_list:
                    author_list.append(author)

    source_branch.unlock()
    target_branch.unlock()
    return author_list

def get_votes(mp):
    ret = ''
    for vote in mp.votes:
        if vote.comment and vote.comment.vote == 'Approve':
            ret=ret+', '+vote.reviewer.display_name
    ret = ret.replace(', ','',1)
    return ret

def get_fixes(mp):
    ret = ''
    ids = ''
    for b in mp.source_branch.linked_bugs_collection:
        ret = ret+', '+b.web_link
        ids = ids+' --fixes lp:'+str(b.id)
    ret = ret.replace(', ','',1)
    return ret,ids

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "b:m:",['branch=','merge-proposal='])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)

    branch = None
    merge_proposal = None
    for o, a in opts:
        if o in ('-b','--branch'):
            branch = a
        elif o in ('-m','--merge-proposal'):
            merge_proposal = a
        else:
            assert False, "unhandled option"

    launchpad = LaunchpadAgent()

    if not merge_proposal: 
        print 'Error: you must specify merge proposal'
        usage()
        sys.exit(2)

    if not branch: 
        #if branch was not specified try to get it from merge proposal
        branch = launchpad.get_branch_from_mp(merge_proposal)
        if not branch:
            print 'Error: unable to get branch from merge proposal.\n\
                            Please specify manually using -- branch.'
            usage()
            sys.exit(2)


    mp = launchpad.get_merge_proposal(branch, merge_proposal)

    if not mp:
        print 'Error: merge proposal relted to this branch was not found'
        sys.exit(1)
    authors = get_authors(mp)
    authors_text=''
    for author in authors:
        authors_text = authors_text+' --author "'+author.replace('"','\\"').replace("'","\\'")+'"'
    if mp.commit_message:
        print 'commit_message="'+mp.commit_message.replace('"','\\"')+'"'
    else:
        print 'commit_message='
    urls,ids = get_fixes(mp)
    print 'launchpad_user="'+LAUNCHPAD_LOGIN+'"'
    print 'bugs_fixed="'+urls+'"'
    print 'approved_by="'+get_votes(mp)+'"'
    print "authors='"+authors_text+"'"
    print 'bugs_fixed_ids="'+ids+'"'
    sys.exit(0)

if __name__ == "__main__":
    main()
