from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import UnencryptedFileCredentialStore
from credentials import AuthorizeRequestTokenWithConsole
from mergeproposalreview import MergeProposalReview
from settings import *
import jenkins
import re



class LaunchpadAgent():
    _launchpad=None

    def __init__(self):
        self._launchpad = self.login()

    def change_mp_status(self, mp, status, message, subject, 
                         revision, vote=None):
        mp.setStatus(status=status, revid=revision)
        if vote:
            mp.createComment(review_type=LAUNCHPAD_REVIEW_TYPE, 
                             subject=subject, content=message,vote=vote)
            review = MergeProposalReview(mp)
            review.unlock()
        else:
            mp.createComment(review_type=LAUNCHPAD_REVIEW_TYPE, 
                             subject=subject, content=message)


    def get_merge_proposal(self, branch_name, merge_proposal_link):
        lp_branch = branch_name.replace('https://code.launchpad.net/','lp:').replace('https://code.staging.launchpad.net/','lp://staging/')
        print 'DEBUG: fetching branch: '+lp_branch
        branch = self._launchpad.branches.getByUrl(url=lp_branch)
        if not branch:
            print 'DEBUG: branch "'+branch_name+'" doesn\'t exist'
            return None

        print 'DEBUG: mp_link: '+merge_proposal_link
    
        for mp in branch.landing_targets:
            print 'DEBUG: mp.web_link: '+mp.web_link
            if mp.web_link == merge_proposal_link:
                return mp
    
        return None


    def approve_mp(self, mp, revision, build_url):
        state = 'PASSED: Continuous integration, rev:'+str(revision)
        print 'DEBUG: '+state
        mp.createComment(review_type=LAUNCHPAD_REVIEW_TYPE,vote='Approve',subject=state,content=state+"\n"+build_url)
        review = MergeProposalReview(mp)
        review.unlock()

    def disapprove_mp(self, mp, revision, build_url):
        state = 'FAILED: Continuous integration, rev:'+str(revision)
        print 'DEBUG: '+state
        mp.createComment(review_type=LAUNCHPAD_REVIEW_TYPE,vote='Disapprove',subject=state, content=state+"\n"+build_url)
        review = MergeProposalReview(mp)
        review.unlock()


    def close_bugs(self, mp):
        for b in mp.source_branch.linked_bugs_collection:
            for task in b.bug_tasks_collection:
                if task.target.name == mp.target_branch.project.name:
                    task.status = 'Fix Committed'
                    task.lp_save()
    

    def get_launchpad_user(self):
        return self._launchpad.people(LAUNCHPAD_LOGIN)

    @staticmethod
    def get_branch_from_mp(merge_proposal):
        m = re.search('(.*)\+merge/[0-9]+$',merge_proposal)
        if m:
            return m.group(1)
        return None


    @staticmethod
    def hide_jenkins_url(url):
        for ip, replacement in URLS_TO_HIDE:
            url = url.replace(ip, replacement)
        return url
 
    @staticmethod
    def login():
        credential_store = UnencryptedFileCredentialStore(CREDENTIAL_STORE_PATH)
        authorization_engine = AuthorizeRequestTokenWithConsole(LP_ENV,LP_APP)
        return Launchpad.login_with(LP_APP,LP_ENV, credential_store=credential_store,authorization_engine=authorization_engine)



class LaunchpadTrigger():
    _launchpad = None        
    _launchpad_user = None

    def __init__(self):
        self._launchpad = LaunchpadAgent.login()
        self._launchpad_user = self._launchpad.people(LAUNCHPAD_LOGIN)

    def get_launchpad_user(self):
        return self._launchpad_user

    def get_latest_review(self, mp):
        revision = 0
        launchpad_user = self.get_launchpad_user()
        for comment in mp.all_comments:
            if comment.author.name == launchpad_user.name:
                if comment.vote_tag == LAUNCHPAD_REVIEW_TYPE:
                    m = re.search('^(PASSED|FAILED): Continuous integration, rev:(\d+)',comment.message_body)
                    if m and (int(m.group(2)) > revision):
                        revision = int(m.group(2))
        return revision
                                  


    def latest_candidate_validated(self, mp):
        latest_review = self.get_latest_review(mp)
        print 'DEBUG: latest_review is revision: '+str(latest_review)
        if latest_review >= mp.source_branch.revision_count:
            print 'DEBUG: skipping this MP. Current revision: '+str(mp.source_branch.revision_count)
            return True
        return False

    def testing_in_progress(self, mp):
        review = MergeProposalReview(mp)
        if review.is_locked():
            print 'DEBUG: skipping this MP. It is curently being '+\
                  'tested by Jenkins.'
            return True
        return False

    def user_allowed_to_trigger_jobs(self, mp):
        allowed_people = ALLOWED_USERS
        if mp.registrant.name in allowed_people:
            return True
        for membership in mp.registrant.memberships_details:
            if membership.team.name in allowed_people:
                return True
        print 'DEBUG: user "'+mp.registrant.name+'" not allowed to trigger jobs'
        return False



    def start_jenkins_job(self, jenkins_url, jenkins_job, mp):
            j = jenkins.Jenkins(jenkins_url)
            jenkins_params = {
                     'landing_candidate': 'bzr+ssh://'+LAUNCHPAD_LOGIN+
                     '@bazaar.launchpad.net/'+mp.source_branch.unique_name,
                     'merge_proposal' : mp.web_link,
                     'candidate_revision' : mp.source_branch.revision_count
                             }

            job_info = j.get_job_info(jenkins_job)
            if not job_info['buildable']:
                print 'DEBUG: jobs is not buildable (probably disabled)'
                return False

            mp.nominateReviewer(reviewer=self.get_launchpad_user(), 
                                review_type=LAUNCHPAD_REVIEW_TYPE)
            review = MergeProposalReview(mp)
            review.lock()
            try:
                print 'DEBUG: starting job: '+ str(jenkins_params)
                j.build_job(jenkins_job, jenkins_params)
            except jenkins.JenkinsException:
                print 'DEBUG: starting a job failed'
                message = "ERROR: failed to start a jenkins job"
                mp.createComment(review_type=LAUNCHPAD_REVIEW_TYPE,
                                 vote='Disapprove',subject=message, 
                                 content=message)



    def trigger_ci_building(self, mps, jenkins_job, jenkins_url):
        for mp in mps:
    
            if not self.user_allowed_to_trigger_jobs(mp):
                continue
    
            if self.testing_in_progress(mp):
                continue
    
            if self.latest_candidate_validated(mp):
                continue
           
            self.start_jenkins_job(jenkins_url, jenkins_job, mp)
    
        return True


    def unapproved_prerequisite_exists(self, mp):
        prereq = mp.prerequisite_branch
        target_branch = mp.target_branch
        if prereq:
            #x is a merge-proposal object
            #i'm actually not sure what Superseded means; I just took this logic
            #from tarmac
            merges = [x for x in prereq.landing_targets
                      if x.target_branch.web_link == target_branch.web_link and
                      x.queue_status != u'Superseded']
            if len(merges) == 0:
                print 'DEBUG: No proposals of prerequisite branch exists'
                return True
            elif len(merges) > 1:
                print 'DEBUG: Too many proposals exists for prerequisite'
                return True
            elif len(merges) == 1:
                if merges[0].queue_status != u'Merged':
                    print 'DEBUG: Prerequisite not yet merged' 
                    return True
        #TODO
        # skip also branches with changes made after approval
        return False

    def trigger_al_building(self, mps, jenkins_job, jenkins_url):
        for mp in mps:
            if not self.user_allowed_to_trigger_jobs(mp):
                continue
    
            if self.testing_in_progress(mp):
                continue
    
            if self.unapproved_prerequisite_exists(mp):
                print 'DEBUG: unapproved prerequisite exists for '+str(mp)
                continue
    
            print 'DEBUG: going to trigger merge: '+str(mp.source_branch)
            self.start_jenkins_job(jenkins_url, jenkins_job, mp)
    
        return 0


    def get_merge_proposals(self, branch_name, state):
        branch = self._launchpad.branches.getByUrl(url=branch_name)    
        if not branch:
            print 'DEBUG: branch "'+branch_name+'" doesn\'t exist'
            return []
        mps= []
        for mp in branch.landing_candidates:
            if mp.queue_status in state:
                mps.append(mp)
        return mps


