import sys
import unittest
import launchpad
from settings import *
from mock import *



class LaunchpadAgentTestCase(unittest.TestCase):
    launchpadAgent = None

    def setUp(self):
        l = MagicMock(name="launchpadlib.launchpadMock")
        with patch('launchpad.Launchpad',new=l):
            self.launchpadAgent = launchpad.LaunchpadAgent()

    def test_change_mp_status_with_no_vote(self):
        mp = MagicMock()
        self.launchpadAgent.change_mp_status(mp, 'Status', 'Message', 
                        'Subject', revision=1)
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        subject='Subject',
                        content='Message')
        mp.setStatus.assert_called_once_with(status='Status',revid=1)

    def test_change_mp_status_with_approve(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        self.launchpadAgent.change_mp_status(mp, 'Status', 'Message', 
                        'Subject', revision=1,vote='Approve')
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        subject='Subject',
                        content='Message',vote='Approve')
        mp.setStatus.assert_called_once_with(status='Status',revid=1)

    def test_change_mp_status_with_disapprove(self):
        mp = MagicMock()
        mp.web_link = 'http://my-example-url.com'
        self.launchpadAgent.change_mp_status(mp, 'Status', 'Message', 
                        'Subject', revision=1,vote='Disapprove')
        mp.createComment.assert_called_once_with(
                        review_type=LAUNCHPAD_REVIEW_TYPE,
                        subject='Subject',
                        content='Message',vote='Disapprove')
        mp.setStatus.assert_called_once_with(status='Status',revid=1)

