#!/usr/bin/env python

import getopt, sys
from settings import *
from launchpad import LaunchpadAgent



def usage():
    print """usage:
             updateMergeProposal.py 
            """

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "j:b:m:efrv:lco:",['job-url=','branch=','merge-proposal=','merge-failed','push-failed','merged','revision=','landing-failed','commit-message-empty','vote='])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)

    branch = None
    job_url  = ''
    mp_state = None
    merge_proposal = None
    message = None
    revision = None
    vote = None
#    merge_failed = False
#    push_failed = False
#    merged = False
    for o, a in opts:
        if o in ('-j','--job-url'):
            job_url = a
        elif o in ('-b','--branch'):
            branch = a
        elif o in ('-o','--vote'):
            vote = a
        elif o in ('-m','--merge-proposal'):
            merge_proposal = a
        elif o in ('-v','--revision'):
            revision = a
        elif o in ('-e','--merge-failed'):
            mp_state='Needs review'
            message="FAILED: Autolanding.\nMerging failed. More details in the following jenkins job:\n"
            subject="Re: [Merge] autolanding failed (merge failed)"
        elif o in ('-f','--push-failed'):
            mp_state='Needs review'
            message="FAILED: Autolanding.\nPushing failed. More details in the following jenkins job:\n"
            subject="Re: [Merge] autolanding failed (push failed)"
        elif o in ('-l','--landing-failed'):
            mp_state='Needs review'
            message="FAILED: Autolanding.\nMore details in the following jenkins job:\n"
            subject="Re: [Merge] autolanding failed"
        elif o in ('-r','--merged'):
            mp_state='Merged'
            subject="Re: [Merge] branch merged"
            message="PASSED: Autolanding.\nBranch merged."
        elif o in ('-c','--commit-message-empty'):
            mp_state='Needs review'
            subject="Re: [Merge] autolanding failed"
            message="FAILED: Autolanding.\nNo commit message was specified.\n"
        else:
            assert False, "unhandled option"


    launchpad = LaunchpadAgent()


    if not mp_state:
        print 'Error: you must specify one of (--merge-failed, --push-failed, --merged)'
        usage()
        sys.exit(2)
       
    if not merge_proposal: 
        print 'Error: you must specify merge proposal'
        usage()
        sys.exit(2)

    if not branch: 
        #if branch was not specified try to get it from merge proposal
        branch = launchpad.get_branch_from_mp(merge_proposal)
        if not branch:
            print 'Error: unable to get branch from merge proposal.\n\
                            Please specify manually using -- branch.'
            usage()
            sys.exit(2)

    if not revision: 
        print 'Error: you must specify revision'
        usage()
        sys.exit(2)


#    if not job_url: 
#        print 'Error: you must specify jenkins job URL'
#        usage()
#        sys.exit(2)

    mp = launchpad.get_merge_proposal(branch, merge_proposal)

    if not mp:
        print 'Error: merge proposal relted to this branch was not found'
        sys.exit(1)

    job_url = launchpad.hide_jenkins_url(job_url)
   
    if message:
        message = message+job_url

    launchpad.change_mp_status(mp, mp_state, message, subject, revision, vote)
    sys.exit(0)


if __name__ == "__main__":
    main()

