#!/usr/bin/env python

import getopt, sys
import urllib
from settings import *
from launchpad import LaunchpadAgent



def usage():
    print """usage:
             voteOnMergeProposal.py -wa
            """


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "s:u:b:p:r:",['status=','build-url=','branch=','merge-proposal=','revision='])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)

    branch = None
    build_url = None
    merge_proposal = None
    approve = False
    revision = None
    for o, a in opts:
        if o in ('-s','--status'):
            if a == 'PASSED':
                approve = True
            else:
                approve = False
        elif o in ('-u','--build-url'):
            build_url = a
        elif o in ('-r','--revision'):
            revision = a
        elif o in ('-b','--branch'):
            branch = a
        elif o in ('-p','--merge-proposal'):
            merge_proposal = a
        else:
            assert False, "unhandled option"

    launchpad = LaunchpadAgent()

    if not build_url:
        print 'Error: you must specify jenkins build URL related to this vote'
        usage()
        sys.exit(2)
       
    if not revision: 
        print 'Error: you must specify revision related to this vote'
        usage()
        sys.exit(2)

    if not merge_proposal: 
        print 'Error: you must specify merge proposal'
        usage()
        sys.exit(2)

    if not branch: 
        #if branch was not specified try to get it from merge proposal
        branch = launchpad.get_branch_from_mp(merge_proposal)
        if not branch:
            print 'Error: unable to get branch from merge proposal.\n\
                            Please specify manually using -- branch.'
            usage()
            sys.exit(2)

    mp = launchpad.get_merge_proposal(branch, merge_proposal)
    if not mp:
        print 'Error: merge proposal relted to this branch was not found'
        sys.exit(1)

    build_url = launchpad.hide_jenkins_url(build_url)

    if approve:
        launchpad.approve_mp(mp, revision, build_url)
    else:
        launchpad.disapprove_mp(mp, revision, build_url)
    sys.exit(0)

if __name__ == "__main__":
    main()

